@@ -124,6 +124,7 @@ sub do_update {
 	 src => $update_target, dst => $cotarget, check_only => $self->{check_only},
 	 xd => $self->{xd});
     $merge->run ($self->{xd}->get_editor (copath => $copath, path => $path,
+					  store_path => $update_target->path,
 					  check_only => $self->{check_only},
 					  ignore_checksum => 1,
 					  oldroot => $xdroot, newroot => $newroot,
@@ -147,7 +148,7 @@ SVK::Command::Update - Bring changes from repository to checkout copies
 
 =head1 OPTIONS
 
- -r [--revision] arg    : act on revision ARG instead of the head revision
+ -r [--revision] REV	: act on revision REV instead of the head revision
  -N [--non-recursive]   : do not descend recursively
  -C [--check-only]      : try operation but make no changes
  -s [--sync]            : synchronize mirrored sources before update
