@@ -261,7 +261,7 @@ sub giant_lock {
     }
 
     open my ($lock), '>', $self->{giantlock}
-	or die loc("Cannot acquire giant loc %1:%2.\n", $self->{giantlock}, $!);
+	or die loc("Cannot acquire giant lock %1:%2.\n", $self->{giantlock}, $!);
     print $lock $$;
     close $lock;
     $self->{giantlocked} = 1;
@@ -542,13 +542,16 @@ L<SVK::Editor::Merge> when called in array context.
 
 sub get_editor {
     my ($self, %arg) = @_;
-    my ($copath, $path) = @arg{qw/copath path/};
+    my ($copath, $path, $spath) = @arg{qw/copath path store_path/};
+    $spath = $path unless defined $spath;
     my $encoding = $self->{checkout}->get ($copath)->{encoding};
     $path = '' if $path eq '/';
+    $spath = '' if $spath eq '/';
     $encoding = Encode::find_encoding($encoding) if $encoding;
     $arg{get_copath} = sub { to_native ($_[0], 'path', $encoding) if $encoding;
 			     $_[0] = SVK::Target->copath ($copath,  $_[0]) };
     $arg{get_path} = sub { $_[0] = "$path/$_[0]" };
+    $arg{get_store_path} = sub { $_[0] = "$spath/$_[0]" };
     my $storage = SVK::Editor::XD->new (%arg, xd => $self);
 
     return wantarray ? ($storage, $self->xd_storage_cb (%arg)) : $storage;
@@ -761,7 +764,6 @@ Don't generate text deltas in C<apply_textdelta> calls.
 sub depot_delta {
     my ($self, %arg) = @_;
     my @root = map {$_->isa ('SVK::XD::Root') ? $_->[1] : $_} @arg{qw/oldroot newroot/};
-    # XXX: if dir_delta croaks the editor would leak since the baton is not properly destroyed.
     SVN::Repos::dir_delta ($root[0], @{$arg{oldpath}},
 			   $root[1], $arg{newpath},
 			   $arg{editor}, undef,
@@ -974,18 +976,20 @@ sub _node_props {
 
 sub _node_type {
     my $copath = shift;
-    lstat ($copath);
+    my $st = [lstat ($copath)];
     return '' if !-e _;
     unless (-r _) {
 	print loc ("Warning: $copath is unreadable.\n");
 	return;
     }
-    return 'file' if -f _ or is_symlink;
-    return 'directory' if -d _;
+    return ('file', $st) if -f _ or is_symlink;
+    return ('directory', $st) if -d _;
     print loc ("Warning: unsupported node type $copath.\n");
-    return;
+    return ('', $st);
 }
 
+use Fcntl ':mode';
+
 sub _delta_file {
     my ($self, %arg) = @_;
     my $pool = SVN::Pool->new_default (undef);
@@ -1003,6 +1007,13 @@ sub _delta_file {
     return 1 if $self->_node_deleted_or_absent (%arg, pool => $pool);
 
     my ($newprops, $fullprops) = $self->_node_props (%arg);
+    if (HAS_SYMLINK && (defined $fullprops->{'svn:special'} xor S_ISLNK($arg{st}[2]))) {
+	# special case obstructure for links, since it's not standard
+	return 1 if $self->_node_deleted_or_absent (%arg,
+						    type => 'link',
+						    pool => $pool);
+	return 1 unless $arg{obstruct_as_replace};
+    }
     my $fh = get_fh ($arg{xdroot}, '<', $arg{path}, $arg{copath}, $fullprops);
     my $mymd5 = md5_fh ($fh);
     my ($baton, $md5);
@@ -1129,7 +1140,7 @@ sub _delta_dir {
 	    next;
 	}
 	next if $unchanged && !$ccinfo->{'.schedule'} && !$ccinfo->{'.conflict'};
-	my $type = _node_type ($copath);
+	my ($type, $st) = _node_type ($copath);
 	next unless defined $type;
 	my $delta = $type ? $type eq 'directory' ? \&_delta_dir : \&_delta_file
 	                  : $kind == $SVN::Node::file ? \&_delta_file : \&_delta_dir;
@@ -1147,6 +1158,7 @@ sub _delta_dir {
 			targets => $newtarget,
 			baton => $baton,
 			root => 0,
+			st => $st,
 			cinfo => $ccinfo,
 			base_path => $arg{base_path} eq '/' ? "/$entry" : "$arg{base_path}/$entry",
 			path => $newpath,
@@ -1213,13 +1225,14 @@ sub _delta_dir {
 	    }
 	    next;
 	}
-	my $type = _node_type ($newpaths{copath}) or next;
+	my ($type, $st) = _node_type ($newpaths{copath}) or next;
 	my $delta = $type eq 'directory' ? \&_delta_dir : \&_delta_file;
 	my $copyfrom = $ccinfo->{'.copyfrom'};
 	my $fromroot = $copyfrom ? $arg{repos}->fs->revision_root ($ccinfo->{'.copyfrom_rev'}) : undef;
 	$self->$delta ( %arg, %newpaths, add => 1, baton => $baton,
 			root => 0, base => 0, cinfo => $ccinfo,
 			type => $type,
+			st => $st,
 			depth => defined $arg{depth} ? defined $targets ? $arg{depth} : $arg{depth} - 1: undef,
 			$copyfrom ?
 			( base => 1,
@@ -1322,6 +1335,15 @@ sub get_eol_layer {
     }
 }
 
+# Remove anything from the keyword value that could prevent us from being able
+# to correctly collapse it again later.
+sub _sanitize_keyword_value {
+    my $value = shift;
+    $value =~ s/[\r\n]/ /g;
+    $value =~ s/ +\$/\$/g;
+    return $value;
+}
+
 sub get_keyword_layer {
     my ($root, $path, $prop) = @_;
     my $k = $prop->{'svn:keywords'};
@@ -1342,18 +1364,16 @@ sub get_keyword_layer {
 		 sub { my ($root, $path) = @_;
 		       my $rev = $root->node_created_rev ($path);
 		       my $fs = $root->fs;
-		       my $author = $fs->revision_prop ($rev, 'svn:author');
-			$author =~ s/[\r\n\$]/ /g;
+		       $fs->revision_prop ($rev, 'svn:author');
 		 },
 		 Id =>
 		 sub { my ($root, $path) = @_;
 		       my $rev = $root->node_created_rev ($path);
 		       my $fs = $root->fs;
-		       my $id = join( ' ', $path, $rev,
+		       join( ' ', $path, $rev,
 			     $fs->revision_prop ($rev, 'svn:date'),
 			     $fs->revision_prop ($rev, 'svn:author'), ''
 			   );
-			$id =~ s/[\r\n\$]/ /g;
 		   },
 		 URL =>
 		 sub { my ($root, $path) = @_;
@@ -1395,7 +1415,7 @@ sub get_keyword_layer {
 
     return PerlIO::via::dynamic->new
 	(translate =>
-         sub { $_[1] =~ s/\$($keyword)(?:: .*? )?\$/"\$$1: ".$kmap{$1}->($root, $path).' $'/eg; },
+         sub { $_[1] =~ s/\$($keyword)(?:: .*? )?\$/"\$$1: "._sanitize_keyword_value($kmap{$1}->($root, $path)).' $'/eg; },
 	 untranslate =>
 	 sub { $_[1] =~ s/\$($keyword)(?:: .*? )?\$/\$$1\$/g; });
 }
@@ -1649,19 +1669,15 @@ sub new {
     bless [@arg], $class;
 }
 
-# XXX: workaround some stalled refs in svn/perl
-my $globaldestroy;
-
 sub DESTROY {
-    warn "===> attempt to destroy root $_[0], leaked?" if $globaldestroy;
-    return if $globaldestroy;
+    return unless $_[0][0];
+    # if this destructor is called upon the pool cleanup which holds the
+    # txn also, we need to use a new pool, otherwise it segfaults for
+    # doing allocation in a pool that is being destroyed.
+    my $pool = SVN::Pool->new_default;
     $_[0][0]->abort if $_[0][0];
 }
 
-END {
-    $globaldestroy = 1;
-}
-
 =head1 AUTHORS
 
 Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
